@@ -1,29 +1,30 @@
-/* $PostgreSQL: pgsql/contrib/ltree/ltree.h,v 1.17 2006/10/04 00:29:45 momjian Exp $ */
+/* $PostgreSQL: pgsql/contrib/ltree/ltree.h,v 1.22 2009/06/11 14:48:51 momjian Exp $ */
 
 #ifndef __LTREE_H__
 #define __LTREE_H__
 
 #include "postgres.h"
-
-#include "utils/builtins.h"
+#include "fmgr.h"
+#include "tsearch/ts_locale.h"
+#include "utils/memutils.h"
 
 typedef struct
 {
-	uint8		len;
+	uint16		len;
 	char		name[1];
-}	ltree_level;
+} ltree_level;
 
-#define LEVEL_HDRSIZE	(sizeof(uint8))
+#define LEVEL_HDRSIZE	(offsetof(ltree_level,name))
 #define LEVEL_NEXT(x)	( (ltree_level*)( ((char*)(x)) + MAXALIGN(((ltree_level*)(x))->len + LEVEL_HDRSIZE) ) )
 
 typedef struct
 {
-	int32		len;
+	int32		vl_len_;		/* varlena header (do not touch directly!) */
 	uint16		numlevel;
 	char		data[1];
-}	ltree;
+} ltree;
 
-#define LTREE_HDRSIZE	MAXALIGN( sizeof(int32) + sizeof(uint16) )
+#define LTREE_HDRSIZE	MAXALIGN( offsetof(ltree, data) )
 #define LTREE_FIRST(x)	( (ltree_level*)( ((char*)(x))+LTREE_HDRSIZE ) )
 
 
@@ -32,12 +33,12 @@ typedef struct
 typedef struct
 {
 	int4		val;
-	uint8		len;
+	uint16		len;
 	uint8		flag;
 	char		name[1];
-}	lquery_variant;
+} lquery_variant;
 
-#define LVAR_HDRSIZE   MAXALIGN(sizeof(uint8)*2 + sizeof(int4))
+#define LVAR_HDRSIZE   MAXALIGN(offsetof(lquery_variant, name))
 #define LVAR_NEXT(x)	( (lquery_variant*)( ((char*)(x)) + MAXALIGN(((lquery_variant*)(x))->len) + LVAR_HDRSIZE ) )
 
 #define LVAR_ANYEND 0x01
@@ -52,9 +53,9 @@ typedef struct
 	uint16		low;
 	uint16		high;
 	char		variants[1];
-}	lquery_level;
+} lquery_level;
 
-#define LQL_HDRSIZE MAXALIGN( sizeof(uint16)*5 )
+#define LQL_HDRSIZE MAXALIGN( offsetof(lquery_level,variants) )
 #define LQL_NEXT(x) ( (lquery_level*)( ((char*)(x)) + MAXALIGN(((lquery_level*)(x))->totallen) ) )
 #define LQL_FIRST(x)	( (lquery_variant*)( ((char*)(x))+LQL_HDRSIZE ) )
 
@@ -68,19 +69,19 @@ typedef struct
 
 typedef struct
 {
-	int32		len;
+	int32		vl_len_;		/* varlena header (do not touch directly!) */
 	uint16		numlevel;
 	uint16		firstgood;
 	uint16		flag;
 	char		data[1];
-}	lquery;
+} lquery;
 
-#define LQUERY_HDRSIZE	 MAXALIGN( sizeof(int32) + 3*sizeof(uint16) )
+#define LQUERY_HDRSIZE	 MAXALIGN( offsetof(lquery, data) )
 #define LQUERY_FIRST(x)   ( (lquery_level*)( ((char*)(x))+LQUERY_HDRSIZE ) )
 
 #define LQUERY_HASNOT		0x01
 
-#define ISALNUM(x)	( isalnum((unsigned char)(x)) || (x) == '_' )
+#define ISALNUM(x)	( t_isalpha(x) || t_isdigit(x)	|| ( pg_mblen(x) == 1 && t_iseq((x), '_') ) )
 
 /* full text query */
 
@@ -97,7 +98,7 @@ typedef struct ITEM
 	/* user-friendly value */
 	uint8		length;
 	uint16		distance;
-}	ITEM;
+} ITEM;
 
 /*
  *Storage:
@@ -105,13 +106,15 @@ typedef struct ITEM
  */
 typedef struct
 {
-	int4		len;
+	int32		vl_len_;		/* varlena header (do not touch directly!) */
 	int4		size;
 	char		data[1];
-}	ltxtquery;
+} ltxtquery;
 
-#define HDRSIZEQT		MAXALIGN( 2*sizeof(int4) )
+#define HDRSIZEQT		MAXALIGN(VARHDRSZ + sizeof(int4))
 #define COMPUTESIZE(size,lenofoperand)	( HDRSIZEQT + (size) * sizeof(ITEM) + (lenofoperand) )
+#define LTXTQUERY_TOO_BIG(size,lenofoperand) \
+	((size) > (MaxAllocSize - HDRSIZEQT - (lenofoperand)) / sizeof(ITEM))
 #define GETQUERY(x)  (ITEM*)( (char*)(x)+HDRSIZEQT )
 #define GETOPERAND(x)	( (char*)GETQUERY(x) + ((ltxtquery*)x)->size * sizeof(ITEM) )
 
@@ -153,14 +156,15 @@ Datum		ltree_textadd(PG_FUNCTION_ARGS);
 /* Util function */
 Datum		ltree_in(PG_FUNCTION_ARGS);
 
-bool ltree_execute(ITEM * curitem, void *checkval,
-			  bool calcnot, bool (*chkcond) (void *checkval, ITEM * val));
+bool ltree_execute(ITEM *curitem, void *checkval,
+			  bool calcnot, bool (*chkcond) (void *checkval, ITEM *val));
 
-int			ltree_compare(const ltree * a, const ltree * b);
-bool		inner_isparent(const ltree * c, const ltree * p);
-bool compare_subnode(ltree_level * t, char *q, int len,
+int			ltree_compare(const ltree *a, const ltree *b);
+bool		inner_isparent(const ltree *c, const ltree *p);
+bool compare_subnode(ltree_level *t, char *q, int len,
 			int (*cmpptr) (const char *, const char *, size_t), bool anyend);
-ltree	   *lca_inner(ltree ** a, int len);
+ltree	   *lca_inner(ltree **a, int len);
+int			ltree_strncasecmp(const char *a, const char *b, size_t s);
 
 #define PG_GETARG_LTREE(x)	((ltree*)DatumGetPointer(PG_DETOAST_DATUM(PG_GETARG_DATUM(x))))
 #define PG_GETARG_LTREE_COPY(x) ((ltree*)DatumGetPointer(PG_DETOAST_DATUM_COPY(PG_GETARG_DATUM(x))))
@@ -178,10 +182,8 @@ ltree	   *lca_inner(ltree ** a, int len);
 typedef unsigned char BITVEC[SIGLEN];
 typedef unsigned char *BITVECP;
 
-#define LOOPBYTE(a) \
-	for(i=0;i<SIGLEN;i++) {\
-		a;\
-	}
+#define LOOPBYTE \
+			for(i=0;i<SIGLEN;i++)
 
 #define GETBYTE(x,i) ( *( (BITVECP)(x) + (int)( (i) / BITBYTE ) ) )
 #define GETBITBYTE(x,i) ( ((unsigned char)(x)) >> i & 0x01 )
@@ -205,23 +207,23 @@ typedef unsigned char *BITVECP;
 
 typedef struct
 {
-	int4		len;
+	int32		vl_len_;		/* varlena header (do not touch directly!) */
 	uint32		flag;
 	char		data[1];
-}	ltree_gist;
+} ltree_gist;
 
 #define LTG_ONENODE 0x01
 #define LTG_ALLTRUE 0x02
 #define LTG_NORIGHT 0x04
 
-#define LTG_HDRSIZE MAXALIGN( sizeof(int4) + sizeof(uint32) )
+#define LTG_HDRSIZE MAXALIGN(VARHDRSZ + sizeof(uint32))
 #define LTG_SIGN(x) ( (BITVECP)( ((char*)(x))+LTG_HDRSIZE ) )
 #define LTG_NODE(x) ( (ltree*)( ((char*)(x))+LTG_HDRSIZE ) )
 #define LTG_ISONENODE(x) ( ((ltree_gist*)(x))->flag & LTG_ONENODE )
 #define LTG_ISALLTRUE(x) ( ((ltree_gist*)(x))->flag & LTG_ALLTRUE )
 #define LTG_ISNORIGHT(x) ( ((ltree_gist*)(x))->flag & LTG_NORIGHT )
 #define LTG_LNODE(x)	( (ltree*)( ( ((char*)(x))+LTG_HDRSIZE ) + ( LTG_ISALLTRUE(x) ? 0 : SIGLEN ) ) )
-#define LTG_RENODE(x)	( (ltree*)( ((char*)LTG_LNODE(x)) + LTG_LNODE(x)->len) )
+#define LTG_RENODE(x)	( (ltree*)( ((char*)LTG_LNODE(x)) + VARSIZE(LTG_LNODE(x))) )
 #define LTG_RNODE(x)	( LTG_ISNORIGHT(x) ? LTG_LNODE(x) : LTG_RENODE(x) )
 
 #define LTG_GETLNODE(x) ( LTG_ISONENODE(x) ? LTG_NODE(x) : LTG_LNODE(x) )
@@ -235,10 +237,8 @@ typedef struct
 #define ASIGLENBIT (ASIGLEN*BITBYTE)
 typedef unsigned char ABITVEC[ASIGLEN];
 
-#define ALOOPBYTE(a) \
-	for(i=0;i<ASIGLEN;i++) {\
-		a;\
-	}
+#define ALOOPBYTE \
+			for(i=0;i<ASIGLEN;i++)
 
 #define AHASHVAL(val) (((unsigned int)(val)) % ASIGLENBIT)
 #define AHASH(sign, val) SETBIT((sign), AHASHVAL(val))
