@@ -4,57 +4,89 @@
  *	  page utilities routines for the postgres inverted index access method.
  *
  *
- * Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *			$PostgreSQL: pgsql/src/backend/access/gin/ginentrypage.c,v 1.5.2.2 2007/10/29 13:49:51 teodor Exp $
+ *			$PostgreSQL: pgsql/src/backend/access/gin/ginentrypage.c,v 1.21.2.1 2009/10/02 21:14:11 tgl Exp $
  *-------------------------------------------------------------------------
  */
 
 #include "postgres.h"
+
 #include "access/gin.h"
-#include "access/tuptoaster.h"
+#include "storage/bufmgr.h"
+#include "utils/rel.h"
 
 /*
- * forms tuple for entry tree. On leaf page, Index tuple has
- * non-traditional layout. Tuple may contain posting list or
- * root blocknumber of posting tree. Macros GinIsPostingTre: (itup) / GinSetPostingTree(itup, blkno)
+ * Form a tuple for entry tree.
+ *
+ * If the tuple would be too big to be stored, function throws a suitable
+ * error if errorTooBig is TRUE, or returns NULL if errorTooBig is FALSE.
+ *
+ * On leaf pages, Index tuple has non-traditional layout. Tuple may contain
+ * posting list or root blocknumber of posting tree.
+ * Macros: GinIsPostingTree(itup) / GinSetPostingTree(itup, blkno)
  * 1) Posting list
- *		- itup->t_info & INDEX_SIZE_MASK contains size of tuple as usual
+ *		- itup->t_info & INDEX_SIZE_MASK contains total size of tuple as usual
  *		- ItemPointerGetBlockNumber(&itup->t_tid) contains original
  *		  size of tuple (without posting list).
- *		  Macroses: GinGetOrigSizePosting(itup) / GinSetOrigSizePosting(itup,n)
+ *		  Macros: GinGetOrigSizePosting(itup) / GinSetOrigSizePosting(itup,n)
  *		- ItemPointerGetOffsetNumber(&itup->t_tid) contains number
- *		  of elements in posting list (number of heap itempointer)
- *		  Macroses: GinGetNPosting(itup) / GinSetNPosting(itup,n)
- *		- After usual part of tuple there is a posting list
+ *		  of elements in posting list (number of heap itempointers)
+ *		  Macros: GinGetNPosting(itup) / GinSetNPosting(itup,n)
+ *		- After standard part of tuple there is a posting list, ie, array
+ *		  of heap itempointers
  *		  Macros: GinGetPosting(itup)
  * 2) Posting tree
  *		- itup->t_info & INDEX_SIZE_MASK contains size of tuple as usual
  *		- ItemPointerGetBlockNumber(&itup->t_tid) contains block number of
  *		  root of posting tree
- *		- ItemPointerGetOffsetNumber(&itup->t_tid) contains magic number GIN_TREE_POSTING
+ *		- ItemPointerGetOffsetNumber(&itup->t_tid) contains magic number
+ *		  GIN_TREE_POSTING, which distinguishes this from posting-list case
+ *
+ * Attributes of an index tuple are different for single and multicolumn index.
+ * For single-column case, index tuple stores only value to be indexed.
+ * For multicolumn case, it stores two attributes: column number of value
+ * and value.
  */
 IndexTuple
-GinFormTuple(GinState *ginstate, Datum key, ItemPointerData *ipd, uint32 nipd)
+GinFormTuple(Relation index, GinState *ginstate,
+			 OffsetNumber attnum, Datum key,
+			 ItemPointerData *ipd, uint32 nipd, bool errorTooBig)
 {
-	bool		isnull = FALSE;
+	bool		isnull[2] = {FALSE, FALSE};
 	IndexTuple	itup;
+	uint32		newsize;
 
-	itup = index_form_tuple(ginstate->tupdesc, &key, &isnull);
+	if (ginstate->oneCol)
+		itup = index_form_tuple(ginstate->origTupdesc, &key, isnull);
+	else
+	{
+		Datum		datums[2];
+
+		datums[0] = UInt16GetDatum(attnum);
+		datums[1] = key;
+		itup = index_form_tuple(ginstate->tupdesc[attnum - 1], datums, isnull);
+	}
 
 	GinSetOrigSizePosting(itup, IndexTupleSize(itup));
 
 	if (nipd > 0)
 	{
-		uint32		newsize = MAXALIGN(SHORTALIGN(IndexTupleSize(itup)) + sizeof(ItemPointerData) * nipd);
-
-		if (newsize >= INDEX_SIZE_MASK)
-			return NULL;
-
-		if (newsize > TOAST_INDEX_TARGET && nipd > 1)
+		newsize = MAXALIGN(SHORTALIGN(IndexTupleSize(itup)) + sizeof(ItemPointerData) * nipd);
+		if (newsize > Min(INDEX_SIZE_MASK, GinMaxItemSize))
+		{
+			if (errorTooBig)
+				ereport(ERROR,
+						(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
+						 errmsg("index row size %lu exceeds maximum %lu for index \"%s\"",
+								(unsigned long) newsize,
+								(unsigned long) Min(INDEX_SIZE_MASK,
+													GinMaxItemSize),
+								RelationGetRelationName(index))));
 			return NULL;
+		}
 
 		itup = repalloc(itup, newsize);
 
@@ -68,11 +100,56 @@ GinFormTuple(GinState *ginstate, Datum key, ItemPointerData *ipd, uint32 nipd)
 	}
 	else
 	{
+		/*
+		 * Gin tuple without any ItemPointers should be large enough to keep
+		 * one ItemPointer, to prevent inconsistency between
+		 * ginHeapTupleFastCollect and ginEntryInsert called by
+		 * ginHeapTupleInsert.  ginHeapTupleFastCollect forms tuple without
+		 * extra pointer to heap, but ginEntryInsert (called for pending list
+		 * cleanup during vacuum) will form the same tuple with one
+		 * ItemPointer.
+		 */
+		newsize = MAXALIGN(SHORTALIGN(IndexTupleSize(itup)) + sizeof(ItemPointerData));
+		if (newsize > Min(INDEX_SIZE_MASK, GinMaxItemSize))
+		{
+			if (errorTooBig)
+				ereport(ERROR,
+						(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
+						 errmsg("index row size %lu exceeds maximum %lu for index \"%s\"",
+								(unsigned long) newsize,
+								(unsigned long) Min(INDEX_SIZE_MASK,
+													GinMaxItemSize),
+								RelationGetRelationName(index))));
+			return NULL;
+		}
+
 		GinSetNPosting(itup, 0);
 	}
 	return itup;
 }
 
+/*
+ * Sometimes we reduce the number of posting list items in a tuple after
+ * having built it with GinFormTuple.  This function adjusts the size
+ * fields to match.
+ */
+void
+GinShortenTuple(IndexTuple itup, uint32 nipd)
+{
+	uint32		newsize;
+
+	Assert(nipd <= GinGetNPosting(itup));
+
+	newsize = MAXALIGN(SHORTALIGN(GinGetOrigSizePosting(itup)) + sizeof(ItemPointerData) * nipd);
+
+	Assert(newsize <= (itup->t_info & INDEX_SIZE_MASK));
+
+	itup->t_info &= ~INDEX_SIZE_MASK;
+	itup->t_info |= newsize;
+
+	GinSetNPosting(itup, nipd);
+}
+
 /*
  * Entry tree is a "static", ie tuple never deletes from it,
  * so we don't use right bound, we use rightest key instead.
@@ -85,28 +162,20 @@ getRightMostTuple(Page page)
 	return (IndexTuple) PageGetItem(page, PageGetItemId(page, maxoff));
 }
 
-Datum
-ginGetHighKey(GinState *ginstate, Page page)
-{
-	IndexTuple	itup;
-	bool		isnull;
-
-	itup = getRightMostTuple(page);
-
-	return index_getattr(itup, FirstOffsetNumber, ginstate->tupdesc, &isnull);
-}
-
 static bool
 entryIsMoveRight(GinBtree btree, Page page)
 {
-	Datum		highkey;
+	IndexTuple	itup;
 
 	if (GinPageRightMost(page))
 		return FALSE;
 
-	highkey = ginGetHighKey(btree->ginstate, page);
+	itup = getRightMostTuple(page);
 
-	if (compareEntries(btree->ginstate, btree->entryValue, highkey) > 0)
+	if (compareAttEntries(btree->ginstate,
+						  btree->entryAttnum, btree->entryValue,
+						  gintuple_get_attrnum(btree->ginstate, itup),
+						  gin_index_getattr(btree->ginstate, itup)) > 0)
 		return TRUE;
 
 	return FALSE;
@@ -151,11 +220,11 @@ entryLocateEntry(GinBtree btree, GinBtreeStack *stack)
 			result = -1;
 		else
 		{
-			bool		isnull;
-
 			itup = (IndexTuple) PageGetItem(page, PageGetItemId(page, mid));
-			result = compareEntries(btree->ginstate, btree->entryValue,
-									index_getattr(itup, FirstOffsetNumber, btree->ginstate->tupdesc, &isnull));
+			result = compareAttEntries(btree->ginstate,
+									   btree->entryAttnum, btree->entryValue,
+								 gintuple_get_attrnum(btree->ginstate, itup),
+								   gin_index_getattr(btree->ginstate, itup));
 		}
 
 		if (result == 0)
@@ -214,13 +283,13 @@ entryLocateLeafEntry(GinBtree btree, GinBtreeStack *stack)
 	while (high > low)
 	{
 		OffsetNumber mid = low + ((high - low) / 2);
-		bool		isnull;
 		int			result;
 
 		itup = (IndexTuple) PageGetItem(page, PageGetItemId(page, mid));
-		result = compareEntries(btree->ginstate, btree->entryValue,
-								index_getattr(itup, FirstOffsetNumber, btree->ginstate->tupdesc, &isnull));
-
+		result = compareAttEntries(btree->ginstate,
+								   btree->entryAttnum, btree->entryValue,
+								 gintuple_get_attrnum(btree->ginstate, itup),
+								   gin_index_getattr(btree->ginstate, itup));
 		if (result == 0)
 		{
 			stack->off = mid;
@@ -354,12 +423,11 @@ entryPlaceToPage(GinBtree btree, Buffer buf, OffsetNumber off, XLogRecData **prd
 	static XLogRecData rdata[3];
 	OffsetNumber placed;
 	static ginxlogInsert data;
-	int 	cnt=0;
 
 	*prdata = rdata;
 	data.updateBlkno = entryPreparePage(btree, page, off);
 
-	placed = PageAddItem(page, (Item) btree->entry, IndexTupleSize(btree->entry), off, LP_USED);
+	placed = PageAddItem(page, (Item) btree->entry, IndexTupleSize(btree->entry), off, false, false);
 	if (placed != off)
 		elog(ERROR, "failed to add item to index page in \"%s\"",
 			 RelationGetRelationName(btree->index));
@@ -372,33 +440,26 @@ entryPlaceToPage(GinBtree btree, Buffer buf, OffsetNumber off, XLogRecData **prd
 	data.isData = false;
 	data.isLeaf = GinPageIsLeaf(page) ? TRUE : FALSE;
 
-    /*
-	 * Prevent full page write if child's split occurs. That is needed
-	 * to remove incomplete splits while replaying WAL
-	 *
-	 * data.updateBlkno contains new block number (of newly created right page)
-	 * for recently splited page.
+	/*
+	 * For incomplete-split tracking, we need updateBlkno information and the
+	 * inserted item even when we make a full page image of the page, so put
+	 * the buffer reference in a separate XLogRecData entry.
 	 */
-	if ( data.updateBlkno == InvalidBlockNumber ) 
-	{
-		rdata[0].buffer = buf;
-		rdata[0].buffer_std = TRUE;
-		rdata[0].data = NULL;
-		rdata[0].len = 0;
-		rdata[0].next = &rdata[1];
-		cnt++;
-	}
+	rdata[0].buffer = buf;
+	rdata[0].buffer_std = TRUE;
+	rdata[0].data = NULL;
+	rdata[0].len = 0;
+	rdata[0].next = &rdata[1];
 
-	rdata[cnt].buffer = InvalidBuffer;
-	rdata[cnt].data = (char *) &data;
-	rdata[cnt].len = sizeof(ginxlogInsert);
-	rdata[cnt].next = &rdata[cnt+1];
-	cnt++;
+	rdata[1].buffer = InvalidBuffer;
+	rdata[1].data = (char *) &data;
+	rdata[1].len = sizeof(ginxlogInsert);
+	rdata[1].next = &rdata[2];
 
-	rdata[cnt].buffer = InvalidBuffer;
-	rdata[cnt].data = (char *) btree->entry;
-	rdata[cnt].len = IndexTupleSize(btree->entry);
-	rdata[cnt].next = NULL;
+	rdata[2].buffer = InvalidBuffer;
+	rdata[2].data = (char *) btree->entry;
+	rdata[2].len = IndexTupleSize(btree->entry);
+	rdata[2].next = NULL;
 
 	btree->entry = NULL;
 }
@@ -450,7 +511,7 @@ entrySplitPage(GinBtree btree, Buffer lbuf, Buffer rbuf, OffsetNumber off, XLogR
 				leftrightmost = NULL;
 	static ginxlogSplit data;
 	Page		page;
-	Page		lpage = GinPageGetCopyPage(BufferGetPage(lbuf));
+	Page		lpage = PageGetTempPageCopy(BufferGetPage(lbuf));
 	Page		rpage = BufferGetPage(rbuf);
 	Size		pageSize = PageGetPageSize(lpage);
 
@@ -511,7 +572,7 @@ entrySplitPage(GinBtree btree, Buffer lbuf, Buffer rbuf, OffsetNumber off, XLogR
 			lsize += MAXALIGN(IndexTupleSize(itup)) + sizeof(ItemIdData);
 		}
 
-		if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, LP_USED) == InvalidOffsetNumber)
+		if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, false, false) == InvalidOffsetNumber)
 			elog(ERROR, "failed to add item to index page in \"%s\"",
 				 RelationGetRelationName(btree->index));
 		ptr += MAXALIGN(IndexTupleSize(itup));
@@ -575,16 +636,16 @@ entryFillRoot(GinBtree btree, Buffer root, Buffer lbuf, Buffer rbuf)
 	page = BufferGetPage(root);
 
 	itup = ginPageGetLinkItup(lbuf);
-	if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, LP_USED) == InvalidOffsetNumber)
+	if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, false, false) == InvalidOffsetNumber)
 		elog(ERROR, "failed to add item to index root page");
 
 	itup = ginPageGetLinkItup(rbuf);
-	if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, LP_USED) == InvalidOffsetNumber)
+	if (PageAddItem(page, (Item) itup, IndexTupleSize(itup), InvalidOffsetNumber, false, false) == InvalidOffsetNumber)
 		elog(ERROR, "failed to add item to index root page");
 }
 
 void
-prepareEntryScan(GinBtree btree, Relation index, Datum value, GinState *ginstate)
+prepareEntryScan(GinBtree btree, Relation index, OffsetNumber attnum, Datum value, GinState *ginstate)
 {
 	memset(btree, 0, sizeof(GinBtreeData));
 
@@ -600,6 +661,7 @@ prepareEntryScan(GinBtree btree, Relation index, Datum value, GinState *ginstate
 
 	btree->index = index;
 	btree->ginstate = ginstate;
+	btree->entryAttnum = attnum;
 	btree->entryValue = value;
 
 	btree->isDelete = FALSE;
