@@ -9,19 +9,18 @@
  * context's MemoryContextMethods struct.
  *
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/mmgr/mcxt.c,v 1.53 2004/12/31 22:02:48 pgsql Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/mmgr/mcxt.c,v 1.66 2009/01/01 17:23:53 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 
 #include "postgres.h"
 
-#include "nodes/memnodes.h"
 #include "utils/memutils.h"
 
 
@@ -47,10 +46,11 @@ MemoryContext MessageContext = NULL;
 MemoryContext TopTransactionContext = NULL;
 MemoryContext CurTransactionContext = NULL;
 
-/* These two are transient links to contexts owned by other objects: */
-MemoryContext QueryContext = NULL;
+/* This is a transient link to the active portal's memory context: */
 MemoryContext PortalContext = NULL;
 
+static void MemoryContextStatsInternal(MemoryContext context, int level);
+
 
 /*****************************************************************************
  *	  EXPORTED ROUTINES														 *
@@ -68,7 +68,8 @@ MemoryContext PortalContext = NULL;
  * In normal multi-backend operation, this is called once during
  * postmaster startup, and not at all by individual backend startup
  * (since the backends inherit an already-initialized context subsystem
- * by virtue of being forked off the postmaster).
+ * by virtue of being forked off the postmaster).  But in an EXEC_BACKEND
+ * build, each process must do this for itself.
  *
  * In a standalone backend this must be called during backend startup.
  */
@@ -78,8 +79,8 @@ MemoryContextInit(void)
 	AssertState(TopMemoryContext == NULL);
 
 	/*
-	 * Initialize TopMemoryContext as an AllocSetContext with slow growth
-	 * rate --- we don't really expect much to be allocated in it.
+	 * Initialize TopMemoryContext as an AllocSetContext with slow growth rate
+	 * --- we don't really expect much to be allocated in it.
 	 *
 	 * (There is special-case code in MemoryContextCreate() for this call.)
 	 */
@@ -90,18 +91,21 @@ MemoryContextInit(void)
 											 8 * 1024);
 
 	/*
-	 * Not having any other place to point CurrentMemoryContext, make it
-	 * point to TopMemoryContext.  Caller should change this soon!
+	 * Not having any other place to point CurrentMemoryContext, make it point
+	 * to TopMemoryContext.  Caller should change this soon!
 	 */
 	CurrentMemoryContext = TopMemoryContext;
 
 	/*
-	 * Initialize ErrorContext as an AllocSetContext with slow growth rate
-	 * --- we don't really expect much to be allocated in it. More to the
-	 * point, require it to contain at least 8K at all times. This is the
-	 * only case where retained memory in a context is *essential* --- we
-	 * want to be sure ErrorContext still has some memory even if we've
-	 * run out elsewhere!
+	 * Initialize ErrorContext as an AllocSetContext with slow growth rate ---
+	 * we don't really expect much to be allocated in it. More to the point,
+	 * require it to contain at least 8K at all times. This is the only case
+	 * where retained memory in a context is *essential* --- we want to be
+	 * sure ErrorContext still has some memory even if we've run out
+	 * elsewhere!
+	 *
+	 * This should be the last step in this function, as elog.c assumes memory
+	 * management works once ErrorContext is non-null.
 	 */
 	ErrorContext = AllocSetContextCreate(TopMemoryContext,
 										 "ErrorContext",
@@ -123,7 +127,10 @@ MemoryContextReset(MemoryContext context)
 {
 	AssertArg(MemoryContextIsValid(context));
 
-	MemoryContextResetChildren(context);
+	/* save a function call in common case where there are no children */
+	if (context->firstchild != NULL)
+		MemoryContextResetChildren(context);
+
 	(*context->methods->reset) (context);
 }
 
@@ -166,9 +173,9 @@ MemoryContextDelete(MemoryContext context)
 	MemoryContextDeleteChildren(context);
 
 	/*
-	 * We delink the context from its parent before deleting it, so that
-	 * if there's an error we won't have deleted/busted contexts still
-	 * attached to the context tree.  Better a leak than a crash.
+	 * We delink the context from its parent before deleting it, so that if
+	 * there's an error we won't have deleted/busted contexts still attached
+	 * to the context tree.  Better a leak than a crash.
 	 */
 	if (context->parent)
 	{
@@ -205,8 +212,8 @@ MemoryContextDeleteChildren(MemoryContext context)
 	AssertArg(MemoryContextIsValid(context));
 
 	/*
-	 * MemoryContextDelete will delink the child from me, so just iterate
-	 * as long as there is a child.
+	 * MemoryContextDelete will delink the child from me, so just iterate as
+	 * long as there is a child.
 	 */
 	while (context->firstchild != NULL)
 		MemoryContextDelete(context->firstchild);
@@ -319,17 +326,22 @@ MemoryContextIsEmpty(MemoryContext context)
  */
 void
 MemoryContextStats(MemoryContext context)
+{
+	MemoryContextStatsInternal(context, 0);
+}
+
+static void
+MemoryContextStatsInternal(MemoryContext context, int level)
 {
 	MemoryContext child;
 
 	AssertArg(MemoryContextIsValid(context));
 
-	(*context->methods->stats) (context);
+	(*context->methods->stats) (context, level);
 	for (child = context->firstchild; child != NULL; child = child->nextchild)
-		MemoryContextStats(child);
+		MemoryContextStatsInternal(child, level + 1);
 }
 
-
 /*
  * MemoryContextCheck
  *		Check all chunks in the named context.
@@ -381,9 +393,9 @@ MemoryContextContains(MemoryContext context, void *pointer)
 		((char *) pointer - STANDARDCHUNKHEADERSIZE);
 
 	/*
-	 * If the context link doesn't match then we certainly have a
-	 * non-member chunk.  Also check for a reasonable-looking size as
-	 * extra guard against being fooled by bogus pointers.
+	 * If the context link doesn't match then we certainly have a non-member
+	 * chunk.  Also check for a reasonable-looking size as extra guard against
+	 * being fooled by bogus pointers.
 	 */
 	if (header->context == context && AllocSizeIsValid(header->size))
 		return true;
@@ -619,7 +631,13 @@ repalloc(void *pointer, Size size)
 /*
  * MemoryContextSwitchTo
  *		Returns the current context; installs the given context.
+ *
+ * This is inlined when using GCC.
+ *
+ * TODO: investigate supporting inlining for some non-GCC compilers.
  */
+#ifndef __GNUC__
+
 MemoryContext
 MemoryContextSwitchTo(MemoryContext context)
 {
@@ -631,6 +649,7 @@ MemoryContextSwitchTo(MemoryContext context)
 	CurrentMemoryContext = context;
 	return old;
 }
+#endif   /* ! __GNUC__ */
 
 /*
  * MemoryContextStrdup
@@ -649,13 +668,28 @@ MemoryContextStrdup(MemoryContext context, const char *string)
 	return nstr;
 }
 
+/*
+ * pnstrdup
+ *		Like pstrdup(), but append null byte to a
+ *		not-necessarily-null-terminated input string.
+ */
+char *
+pnstrdup(const char *in, Size len)
+{
+	char	   *out = palloc(len + 1);
+
+	memcpy(out, in, len);
+	out[len] = '\0';
+	return out;
+}
+
 
 #if defined(WIN32) || defined(__CYGWIN__)
 /*
  *	Memory support routines for libpgport on Win32
  *
- *	Win32 can't load a library that DLLIMPORTs a variable
- *	if the link object files also DLLIMPORT the same variable.
+ *	Win32 can't load a library that PGDLLIMPORTs a variable
+ *	if the link object files also PGDLLIMPORT the same variable.
  *	For this reason, libpgport can't reference CurrentMemoryContext
  *	in the palloc macro calls.
  *
@@ -676,7 +710,7 @@ pgport_pstrdup(const char *str)
 }
 
 
-/* Doesn't reference a DLLIMPORT variable, but here for completeness. */
+/* Doesn't reference a PGDLLIMPORT variable, but here for completeness. */
 void
 pgport_pfree(void *pointer)
 {
