@@ -8,7 +8,7 @@
  *
  * Copyright (c) 2000-2009, PostgreSQL Global Development Group
  *
- * $PostgreSQL: pgsql/src/bin/psql/describe.c,v 1.218.2.1 2009/07/20 03:46:53 tgl Exp $
+ * $PostgreSQL: pgsql/src/bin/psql/describe.c,v 1.218.2.3 2010/03/01 20:55:53 heikki Exp $
  */
 #include "postgres_fe.h"
 
@@ -415,7 +415,7 @@ describeFunctions(const char *functypes, const char *pattern, bool verbose, bool
 		appendPQExpBuffer(&buf, "      )\n");
 	}
 
-	processSQLNamePattern(pset.db, &buf, pattern, have_where, true,
+	processSQLNamePattern(pset.db, &buf, pattern, have_where, false,
 						  "n.nspname", "p.proname", NULL,
 						  "pg_catalog.pg_function_is_visible(p.oid)");
 
@@ -1259,10 +1259,10 @@ describeOneTableDetails(const char *schemaname,
 	for (i = 0; i < numrows; i++)
 	{
 		/* Column */
-		printTableAddCell(&cont, PQgetvalue(res, i, 0), false);
+		printTableAddCell(&cont, PQgetvalue(res, i, 0), false, false);
 
 		/* Type */
-		printTableAddCell(&cont, PQgetvalue(res, i, 1), false);
+		printTableAddCell(&cont, PQgetvalue(res, i, 1), false, false);
 
 		/* Modifiers: not null and default */
 		if (show_modifiers)
@@ -1283,12 +1283,12 @@ describeOneTableDetails(const char *schemaname,
 			}
 
 			modifiers[i] = pg_strdup(tmpbuf.data);
-			printTableAddCell(&cont, modifiers[i], false);
+			printTableAddCell(&cont, modifiers[i], false, false);
 		}
 
 		/* Value: for sequences only */
 		if (tableinfo.relkind == 'S')
-			printTableAddCell(&cont, seq_values[i], false);
+			printTableAddCell(&cont, seq_values[i], false, false);
 
 		/* Storage and Description */
 		if (verbose)
@@ -1301,8 +1301,8 @@ describeOneTableDetails(const char *schemaname,
 									   (storage[0] == 'x' ? "extended" :
 										(storage[0] == 'e' ? "external" :
 										 "???")))),
-							  false);
-			printTableAddCell(&cont, PQgetvalue(res, i, 6), false);
+							  false, false);
+			printTableAddCell(&cont, PQgetvalue(res, i, 6), false, false);
 		}
 	}
 
@@ -2007,7 +2007,7 @@ describeRoles(const char *pattern, bool verbose)
 
 	for (i = 0; i < nrows; i++)
 	{
-		printTableAddCell(&cont, PQgetvalue(res, i, 0), false);
+		printTableAddCell(&cont, PQgetvalue(res, i, 0), false, false);
 
 		resetPQExpBuffer(&buf);
 		if (strcmp(PQgetvalue(res, i, 1), "t") == 0)
@@ -2042,12 +2042,12 @@ describeRoles(const char *pattern, bool verbose)
 
 		attr[i] = pg_strdup(buf.data);
 
-		printTableAddCell(&cont, attr[i], false);
+		printTableAddCell(&cont, attr[i], false, false);
 
-		printTableAddCell(&cont, PQgetvalue(res, i, 7), false);
+		printTableAddCell(&cont, PQgetvalue(res, i, 7), false, false);
 
 		if (verbose && pset.sversion >= 80200)
-			printTableAddCell(&cont, PQgetvalue(res, i, 8), false);
+			printTableAddCell(&cont, PQgetvalue(res, i, 8), false, false);
 	}
 	termPQExpBuffer(&buf);
 
