@@ -3,12 +3,12 @@
  * path.c
  *	  portable path handling routines
  *
- * Portions Copyright (c) 1996-2008, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/port/path.c,v 1.74.2.2 2009/04/03 23:27:22 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/port/path.c,v 1.79 2009/06/11 14:49:15 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -165,6 +165,8 @@ make_native_path(char *filename)
 /*
  * join_path_components - join two path components, inserting a slash
  *
+ * We omit the slash if either given component is empty.
+ *
  * ret_path is the output area (must be of size MAXPGPATH)
  *
  * ret_path can be the same as head, but not the same as tail.
@@ -177,37 +179,22 @@ join_path_components(char *ret_path,
 		strlcpy(ret_path, head, MAXPGPATH);
 
 	/*
-	 * Remove any leading "." and ".." in the tail component, adjusting head
-	 * as needed.
+	 * Remove any leading "." in the tail component.
+	 *
+	 * Note: we used to try to remove ".." as well, but that's tricky to get
+	 * right; now we just leave it to be done by canonicalize_path() later.
 	 */
-	for (;;)
-	{
-		if (tail[0] == '.' && IS_DIR_SEP(tail[1]))
-		{
-			tail += 2;
-		}
-		else if (tail[0] == '.' && tail[1] == '\0')
-		{
-			tail += 1;
-			break;
-		}
-		else if (tail[0] == '.' && tail[1] == '.' && IS_DIR_SEP(tail[2]))
-		{
-			trim_directory(ret_path);
-			tail += 3;
-		}
-		else if (tail[0] == '.' && tail[1] == '.' && tail[2] == '\0')
-		{
-			trim_directory(ret_path);
-			tail += 2;
-			break;
-		}
-		else
-			break;
-	}
+	while (tail[0] == '.' && IS_DIR_SEP(tail[1]))
+		tail += 2;
+
 	if (*tail)
+	{
+		/* only separate with slash if head wasn't empty */
 		snprintf(ret_path + strlen(ret_path), MAXPGPATH - strlen(ret_path),
-				 "/%s", tail);
+				 "%s%s",
+				 (*(skip_drive(head)) != '\0') ? "/" : "",
+				 tail);
+	}
 }
 
 
@@ -432,7 +419,7 @@ dir_strcmp(const char *s1, const char *s2)
 #ifndef WIN32
 			*s1 != *s2
 #else
-			/* On windows, paths are case-insensitive */
+		/* On windows, paths are case-insensitive */
 			pg_tolower((unsigned char) *s1) != pg_tolower((unsigned char) *s2)
 #endif
 			&& !(IS_DIR_SEP(*s1) && IS_DIR_SEP(*s2)))
@@ -606,6 +593,15 @@ get_doc_path(const char *my_exec_path, char *ret_path)
 	make_relative_path(ret_path, DOCDIR, PGBINDIR, my_exec_path);
 }
 
+/*
+ *	get_html_path
+ */
+void
+get_html_path(const char *my_exec_path, char *ret_path)
+{
+	make_relative_path(ret_path, HTMLDIR, PGBINDIR, my_exec_path);
+}
+
 /*
  *	get_man_path
  */
@@ -656,6 +652,15 @@ get_home_path(char *ret_path)
  *
  * Modify the given string in-place to name the parent directory of the
  * named file.
+ *
+ * If the input is just a file name with no directory part, the result is
+ * an empty string, not ".".  This is appropriate when the next step is
+ * join_path_components(), but might need special handling otherwise.
+ *
+ * Caution: this will not produce desirable results if the string ends
+ * with "..".  For most callers this is not a problem since the string
+ * is already known to name a regular file.  If in doubt, apply
+ * canonicalize_path() first.
  */
 void
 get_parent_directory(char *path)
