@@ -71,7 +71,7 @@ sub Install
     CopyFiles(
         'Import libraries',
         $target .'/lib/',
-        "$conf\\", "postgres\\postgres.lib","libpq\\libpq.lib", "libecpg\\libecpg.lib", "libpgport\\libpgport.lib"
+        "$conf\\", "postgres\\postgres.lib","libpq\\libpq.lib", "libecpg\\libecpg.lib", "libpgport\\libpgport.lib","libpgtypes\\libpgtypes.lib","libecpg_compat\\libecpg_compat.lib"
     );
     CopySetOfFiles('timezone names', 
 				   [ glob('src\timezone\tznames\*.txt') ] ,
